# $Id: libjobdb.py,v 1.11 2007/04/10 21:49:24 kipp Exp $
#
# Copyright (C) 2005  Kipp C. Cannon
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


#
# =============================================================================
#
#                                   Preamble
#
# =============================================================================
#


import sqlobject

from glue import segments
from glue import segmentsUtils
from dagdbUpdator import BOSS

__author = "Kipp Cannon <kipp@gravity.phys.uwm.edu>"
__version__ = "$Revision: 1.11 $"[11:-2]
__date__ = "$Date: 2007/04/10 21:49:24 $"[7:-2]


#
# =============================================================================
#
#                              Query Construction
#
# =============================================================================
#


def IDQuery(idseglist):
	"""
	Construct an sqlobject query object for finding the jobs whose IDs
	are in the ranges described by the glue.segmentlist object
	idseglist.
	"""
	query = False
	for seg in idseglist:
		if (seg[0] == -segments.infinity()) and (seg[1] == segments.infinity()):
			return True
		elif (seg[0] == -segments.infinity()) and (seg[1] != segments.infinity()):
			query |= BOSS.BossJob.q.id <= seg[1]
		elif (seg[0] != -segments.infinity()) and (seg[1] == segments.infinity()):
			query |= BOSS.BossJob.q.id >= seg[0]
		else:
			query |= (BOSS.BossJob.q.id >= seg[0]) & (BOSS.BossJob.q.id <= seg[1])
	return query


def UserQuery(user):
	"""
	Construct an sqlobject query object for finding the jobs submitted
	by user.
	"""
	return BOSS.BossJob.q.s_usr == str(user)


def StateQuery(state):
	"""
	Construct an sqlobject query object for finding jobs in the
	specified state.  state is one of "wait", "idle", "run", "abort",
	"stop", "success", "fail".
	"""
	if state == "wait":
		return BOSS.BossJob.q.t_stat == ""
	elif state == "idle":
		return BOSS.BossJob.q.t_stat == "I"
	elif state == "run":
		return BOSS.BossJob.q.t_stat == "R"
	elif state == "abort":
		return BOSS.BossJob.q.t_stat == "A"
	elif state == "stop":
		return (BOSS.BossJob.q.t_stat == "T") & (BOSS.BossJob.q.ret_code == "")
	elif state == "success":
		return (BOSS.BossJob.q.t_stat == "T") & (BOSS.BossJob.q.ret_code == "0")
	elif state == "fail":
		return (BOSS.BossJob.q.t_stat == "T") & (BOSS.BossJob.q.ret_code != "") & (BOSS.BossJob.q.ret_code != "0")
	else:
		raise ValueError, state


def SubmitTimeQuery(timeseglist):
	"""
	Construct an sqlobject query object for finding jobs whose submit
	time lies in the intervals described by the glue.segmentlist object
	timeseglist.
	"""
	query = False
	for seg in timeseglist:
		if (seg[0] == -segments.infinity()) and (seg[1] == segments.infinity()):
			return True
		elif (seg[0] == -segments.infinity()) and (seg[1] != segments.infinity()):
			query |= BOSS.BossJob.q.t_sub <= seg[1]
		elif (seg[0] != -segments.infinity()) and (seg[1] == segments.infinity()):
			query |= BOSS.BossJob.q.t_sub >= seg[0]
		else:
			query |= (BOSS.BossJob.q.t_sub >= seg[0]) & (BOSS.BossJob.q.t_sub <= seg[1])
	return query


def StartTimeQuery(timeseglist):
	"""
	Construct an sqlobject query object for finding jobs whose start
	time lies in the intervals described by the glue.segmentlist object
	timeseglist.
	"""
	query = False
	for seg in timeseglist:
		if (seg[0] == -segments.infinity()) and (seg[1] == segments.infinity()):
			return True
		elif (seg[0] == -segments.infinity()) and (seg[1] != segments.infinity()):
			query |= BOSS.BossJob.q.t_start <= seg[1]
		elif (seg[0] != -segments.infinity()) and (seg[1] == segments.infinity()):
			query |= BOSS.BossJob.q.t_start >= seg[0]
		else:
			query |= (BOSS.BossJob.q.t_start >= seg[0]) & (BOSS.BossJob.q.t_start <= seg[1])
	return query


def StopTimeQuery(timeseglist):
	"""
	Construct an sqlobject query object for finding jobs whose stop
	time lies in the intervals described by the glue.segmentlist object
	timeseglist.
	"""
	query = False
	for seg in timeseglist:
		if (seg[0] == -segments.infinity()) and (seg[1] == segments.infinity()):
			return True
		elif (seg[0] == -segments.infinity()) and (seg[1] != segments.infinity()):
			query |= BOSS.BossJob.q.t_stop <= seg[1]
		elif (seg[0] != -segments.infinity()) and (seg[1] == segments.infinity()):
			query |= BOSS.BossJob.q.t_stop >= seg[0]
		else:
			query |= (BOSS.BossJob.q.t_stop >= seg[0]) & (BOSS.BossJob.q.t_stop <= seg[1])
	return query


#
# =============================================================================
#
#                             Application Helpers
#
# =============================================================================
#


def SegsToRanges(seglist):
	"""
	Turn a segment list into a list of range strings as could be parsed by
	ParseRanges().
	"""
	# FIXME: remove this wrapping, just call the function directly
	return segmentsUtils.to_range_strings(seglist)


def ParseRanges(ranges, boundtype = int):
	"""
	Parse a list of ranges expressed as strings in the form "value" or
	"first:last" into an equivalent glue.segments.segmentlist.  In the
	latter case, an empty string for "first" and(or) "last" indicates a
	(semi)infinite range.
	"""
	# FIXME: remove this wrapping, just call the function directly
	return segmentsUtils.from_range_strings(ranges, boundtype = boundtype).coalesce()


def SPathsToIDs(query, paths):
	"""
	Using the jobs found by the given query, for each path in the list
	of paths, determine the ID of the most recent job having that
	submit path (s_path).  The return value is a list of the IDs, in
	order, one for each path, with None for paths for which no matching
	job was found.
	"""
	result = [None] * len(paths)
	found = 0
	for job in BOSS.BossJob.select(query, orderBy="t_sub").reversed():
		try:
			index = paths.index(job.s_path)
		except ValueError:
			continue
		if result[index] != None:
			continue
		result[index] = job.id
		found += 1
		if found == len(paths):
			break
	return result
